package net.sf.sql2java;

import java.io.File;

import net.sf.sql2java.data.holders.Database;
import net.sf.sql2java.fetching.FetchingException;

import org.apache.commons.configuration.CompositeConfiguration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.XMLConfiguration;
import org.apache.log4j.Logger;

/**
 *
 * @author Daan Gerits
 */
public class Launcher {
	protected static final String configFile = "config.cfg.xml";
	protected static final String convertorFile = "conversions.cfg.xml";
	private final Logger LOG = Logger.getLogger(Launcher.class);
	private String path;
	
// -- Constructors ------------------------------------------------------------
    
    public Launcher(String path) {
    	if (! path.endsWith(File.separator))
    		path += File.separator;
    	
    	this.path = path;
    }

// -- Initialization ----------------------------------------------------------
    
    public void start() {
    	try {
    		// -- create the configuration
    		CompositeConfiguration config = new CompositeConfiguration();
    		config.addConfiguration(new XMLConfiguration(path + configFile));
    		config.addConfiguration(new XMLConfiguration(path + convertorFile));
    		
	    	// -- set the configuration
	    	ApplicationContext.getInstance().setConfiguration(config);
	    	
	    	// -- start fetching
	    	Database database = ApplicationContext.getInstance()
	    											 .getFetcher()
	    											 .fetch();
	    	
	    	// -- after fetching succeeded, generation can be started
	    	ApplicationContext.getInstance().getGenerator().generate(database);
	    	
    	} catch (ConfigurationException ce) {
    		LOG.fatal("An error occured while configuring the system!", ce);
    	} catch (FetchingException fe) {
    		LOG.fatal("An error occured while fetching data!", fe);
		}
    }
    
// -- Main Loading ------------------------------------------------------------
    
    public static void main(String[] args) {
    	// -- create the launcher
        Launcher launcher = new Launcher("");

        // -- start the launcher
        launcher.start();
    }
}
 