/*
 * Convertor.java
 *
 * Created on April 5, 2007, 1:27 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.sf.sql2java.common.conversion;

import java.util.HashMap;
import java.util.Map;
import net.sf.sql2java.utils.StringUtils;

/**
 *
 * @author dge2
 */
public class ConversionManager {
    private Map<String, JavaKeyword> javaKeywords;
    private Map<String, SqlKeyword> sqlKeywords;
    private Map<String, DataType> datatypes;
    
// -- Constructors ------------------------------------------------------------
    
    public ConversionManager() {
        javaKeywords = new HashMap<String, JavaKeyword>();
        sqlKeywords = new HashMap<String, SqlKeyword>();
        datatypes = new HashMap<String, DataType>();
    }
    
// -- Implementations ---------------------------------------------------------

    public String convertSqlKeyword(String input) {
        Keyword found = sqlKeywords.get(input.toUpperCase());
        
        if (found != null)
            found.setName(input);
        
        return (found == null) ? input : found.replace();
    }
    
    public String convertJavaKeyword(String input) {
        input = StringUtils.camelcase(input, true, false);
        
        Keyword found = javaKeywords.get(input.toLowerCase());
        
        return (found == null) ? input : found.replace();
    }
    
    public String convertDataType(String sqlDataType) {
        DataType found = datatypes.get(sqlDataType);
        
        return (found == null) ? sqlDataType : found.getJavaType();
    }
    
// -- Testers -----------------------------------------------------------------
    
    public boolean isJavaKeyword(String input) {
        return javaKeywords.containsKey(input.toLowerCase());
    }
    
    public boolean isSqlKeyword(String input) {
        return sqlKeywords.containsKey(input.toUpperCase());
    }
    
// -- Datatype Map Manipulation -----------------------------------------------
    
    public void addDataType(String sqlDataType, String javaDataType) {
        addDataType(new DataType(sqlDataType, javaDataType));
    }
    
    public void addDataType(DataType dataType) {
        datatypes.put(dataType.getSqlType(), dataType);
    }
    
    public void removeDataType(DataType dataType) {
        datatypes.remove(dataType.getSqlType());
    }
    
// -- Keyword Map Manipulation ------------------------------------------------
    
    public void addJavaKeyword(JavaKeyword keyword) {
        javaKeywords.put(keyword.getName().toLowerCase(), keyword);
    }
    
    public void addSqlKeyword(SqlKeyword keyword) {
        sqlKeywords.put(keyword.getName(), keyword);
    }
    
    public void removeKeyword(Keyword keyword) {
        if (keyword.getContext().equalsIgnoreCase("java"))
            javaKeywords.remove(keyword.getName());
        else if (keyword.getContext().equalsIgnoreCase("sql"))
            sqlKeywords.remove(keyword.getName());
    }
}
