package net.sf.sql2java.conversion;

import java.util.HashMap;
import java.util.Map;

import net.sf.sql2java.common.configuration.Configurer;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.log4j.Logger;

public class ConvertorConfigurer extends Configurer<Convertor> {
	private static final Logger LOG = Logger.getLogger(ConvertorConfigurer.class);

// -- Configuration -----------------------------------------------------------
	
	@Override
	public void configure(Convertor subject) 
	throws ConfigurationException {
		LOG.info("Configuring convertor.");
		
		// -- check if the configuration is valid
		if (getConfiguration() == null)
			throw new ConfigurationException("No configuration has been set");
		
		// -- get the keyword groups
		Map<String, KeywordGroup> groups = getKeywordGroups();
		subject.setKeywordGroups(groups);
	}
	
// -- Helpers -----------------------------------------------------------------
	
	protected Map<String, KeywordGroup> getKeywordGroups() {
		// -- create the result
		Map<String, KeywordGroup> result = new HashMap<String, KeywordGroup>();
		
		int size = getConfiguration().getList("keywords.keywordGroup[@name]").size();
		
		for (int i = 0; i < size; i++) {
			KeywordGroup group = createKeywordGroup(i);
			
			result.put(group.getName(), group);
		}
		
		// -- return the result
		return result;
	}

	protected KeywordGroup createKeywordGroup(int i) {
		// -- create the result
		KeywordGroup result = new KeywordGroup();
		
		// -- create the id
		String id = "keywords.keywordGroup(" + i + ")";
		
		// -- get the group parameters
		result.setName(getConfiguration().getString(id + "[@name]"));
		result.setCaseSensitive(getConfiguration().getBoolean(id + "[@caseSensitive]"));
		result.setTransform(getConfiguration().getString(id + "[@transform]"));
		
		int size = getConfiguration().getList("keywords.keywordGroup(" + i + ").keyword[@name]").size();
		
		// -- get the keywords
		for (int j = 0; j < size; j++) {
			result.addKeyword(createKeyword(i, j));
		}
	
		// -- return the result
		return result;
	}
	
	protected Keyword createKeyword(int group, int keyword) {
		// -- create the result
		Keyword result = new Keyword();
		
		// -- create the id
		String id = "keywords.keywordGroup(" + group + ").keyword(" + keyword + ")";
		
		result.setName(getConfiguration().getString(id + "[@name]"));
		
		// -- return the result
		return result;
	}
}
