/*
 * DirectoryPackageReader.java
 * 
 * Created on Jul 11, 2007, 12:50:32 PM
 * 
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.sf.sql2java.packages;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import net.sf.sql2java.generator.scripting.Script;
import net.sf.sql2java.generator.template.Template;

import org.apache.commons.configuration.ConfigurationException;

/**
 *
 * @author dge2
 */
public class DirectoryPackageReader extends PackageReader {

// -- Implementations ---------------------------------------------------------
    
    public TemplatePack readPack(String packPath) throws IOException, ConfigurationException {
        // -- create a file from the provided path
        File file = new File(packPath);
        
        // -- check if the provided directory exists
        if (! file.exists())
            throw new ConfigurationException("Directory does not exists : '" + packPath + "'");
        
        // -- check if the provided path points to a directory
        if (! file.isDirectory())
            throw new ConfigurationException("Provided path doesn't point to a directory");
        
        // -- create the descriptor file
        File descriptor = new File(composeFilename(packPath, "descriptor.xml"));

        // -- check if the directory contains a descriptor
        if (! descriptor.exists())
            throw new ConfigurationException("Invalid Pack Format, need descriptor inside pack");

        // -- read the descriptor
        TemplatePack unInitializedPack = readDescriptor(descriptor);
        
        // -- inject the contents
        TemplatePack result = readResources(unInitializedPack, packPath);
        
        // -- return the result
        return result;
    }

// -- Helping Methods ---------------------------------------------------------
    
    private TemplatePack readResources(TemplatePack unInitializedPack, String directoryName) throws IOException, ConfigurationException {
        // -- get the content of the scripts
        for (Script script : unInitializedPack.getScripts().values()) {
            File file = new File(composeFilename(directoryName, script.getResource()));
            
            if (! file.exists())
                throw new ConfigurationException("Script '" + file.getAbsolutePath() + "' doesn't exists");
            
            String scriptContent = readFile(new FileInputStream(file));
            
            script.setContent(scriptContent);
        }
        
        // -- get the content of the templates
        for (Template template : unInitializedPack.getTemplates().values()) {
            File file = new File(composeFilename(directoryName, template.getResource()));
            
            if (! file.exists())
                throw new ConfigurationException("Template '" + file.getAbsolutePath() + "' doesn't exists");
            
            String templateContent = readFile(new FileInputStream(file));
            
            template.setContent(templateContent);
        }
        
        return unInitializedPack;
    }
        
    private String composeFilename(String directory, String filename) {
        if (! directory.endsWith(File.separator)) 
        	directory += File.separator;
        
        return directory + filename;
    }
}
