/*
 * Pack.java
 * 
 * Created on Jul 9, 2007, 4:19:50 PM
 * 
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.sf.sql2java.packages;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import net.sf.sql2java.generator.scripting.Script;
import net.sf.sql2java.generator.template.Template;

/**
 *
 * @author dge2
 */
public class TemplatePack {
    private String name;
    private String version;
    private String description;
    private Map<String, Template> templates;
    private Map<String, List<Template>> scopedMap;
    private Map<String, Script> scripts;
    
// -- Constructors ------------------------------------------------------------

    public TemplatePack() {
    	scopedMap = new HashMap<String, List<Template>>();
    	templates = new HashMap<String, Template>();
    	scripts = new HashMap<String, Script>();
    }
    
// -- Templates Manipulation --------------------------------------------------
    
    public void addTemplate(Template template) {
        templates.put(template.getName(), template);
        
        if (! scopedMap.containsKey(template.getScope()))
        	scopedMap.put(template.getScope(), new LinkedList<Template>());
        
    	scopedMap.get(template.getScope()).add(template);
    }
    
    public void removeTemplate(Template template) {
    	templates.remove(template.getName());
    	
    	if (! scopedMap.containsKey(template.getScope())) return;
    	
    	scopedMap.get(template.getScope()).remove(template);
    	
    	if (scopedMap.get(template.getScope()).size() == 0)
    		scopedMap.remove(template.getScope());
    }
    
    public Template getTemplate(String name) {
        return templates.get(name);
    }
    
    public List<Template> getTemplates(String scope) {
    	if (! scopedMap.containsKey(scope))
    		return new LinkedList<Template>();
    	else
    		return scopedMap.get(scope);
    }
    
// -- Scripts Manipulation ----------------------------------------------------
    
    public void addScript(Script script) {
        script.setTemplatePack(this);
        
        scripts.put(script.getName(), script);
    }
    
    public void removeScript(Script script) {
        scripts.remove(script.getName());
    }
    
    public Script getScript(String name) {
        return scripts.get(name);
    }
    
// -- Getters -----------------------------------------------------------------
    
    public String getName() {
        return name;
    }
    
    public String getVersion() {
        return version;
    }
    
    public String getDescription() {
        return description;
    }
    
    public Map<String, Template> getTemplates() {
    	return templates;
    }
    
    public Map<String, Script> getScripts() {
        return scripts;
    }
    
// -- Setters -----------------------------------------------------------------
    
    public void setName(String name) {
        this.name = name;
    }
    
    public void setVersion(String version) {
        this.version = version;
    }
    
    public void setDescription(String description) {
        this.description = description;
    }
    
    public void setTemplates(Map<String, Template> templates) {
        for (Template template : templates.values())
        	addTemplate(template);
    }
    
    public void setScripts(Map<String, Script> scripts) {
    	for (Script script : scripts.values())
    		addScript(script);
    }
}
