// ============================================================================
// == PROPERTIES
// ============================================================================

function createProperty(field) {
    if (field.reference == null)
    	return _createProperty(field, false);
   	else
        return _createProperty(field.reference, true);
}

function _createProperty(field, reference) {
    // -- create the result object
    var result = "";
    
    // -- create the parameter String
    naming = util.createParameterString(
        "public static final String PROPERTY_:{identifier@uc} = \":{identifier}\";"
    );
    
    if (reference)
        naming.addParameter("identifier", convert.translateTo("java", field.parent.name));
    else
        naming.addParameter("identifier", convert.translateTo("java", field.name));
    
    // -- add the line to the result after parsing it
    result += naming.parse();
    
    // -- return the result
    return result;
}

// ============================================================================
// == DECLARATION
// ============================================================================

function declaration(field) {
    if (field.reference == null)
    	return _declaration(field, false);
   	else
        return _declaration(field.reference, true);
}

function _declaration(field, reference) {
    // -- create the result object
    var result = "";
    
    naming = util.createParameterString(
        "protected :{type} :{identifier@lc};"
    );
    
    if (reference) {
        // -- get the reference type
        var userClass = script.execute("default.beanClass", field.parent.name);
        
        naming.addParameter("type", userClass);
        naming.addParameter("identifier", convert.translateTo("java", field.parent.name));
    } else {
        naming.addParameter("type", util.getJavaType(field));
        naming.addParameter("identifier", convert.translateTo("java", field.name));
    }
    
    result += naming.parse();
    
    // -- return the result
    return result;
}

// ============================================================================
// == DECLARATION INITIALIZATION FLAG
// ============================================================================

function declarationInit(field) {
    if (field.reference == null)
    	return _declarationInit(field, false);
    else
		return _declarationInit(field.reference, true);        
}

function _declarationInit(field, reference) {
    // -- create the result object
    var result = "";

    naming = util.createParameterString(
        "private boolean :{field@lc}Initialized = false;"
    );

    if (reference)
        naming.addParameter("field", convert.translateTo("java", field.parent.name));
    else
        naming.addParameter("field", convert.translateTo("java", field.name));
    
    // -- add the line to the result after parsing it
    result += naming.parse();
    
    // -- return the result object
    return result;
}

// ============================================================================
// == COPY LINE FOR CLONE METHOD
// ============================================================================

function copyLine(field, bean) {
    if (field.reference != null)
        return _copyLine(field.reference, bean, true);
    else 
        return _copyLine(field, bean, false);
}

function _copyLine(field, bean, reference) {
    // -- create the result
    var result = "";
    
    naming = util.createParameterString(
        ":{bean}.set:{type@lc@ca}(get:{type@lc@ca}());"
    );
    
    if (reference) {
        naming.addParameter("type", convert.translateTo("java", field.parent.name));
    } else {
        naming.addParameter("type", convert.translateTo("java", field.name));
    }
    
    naming.addParameter("bean", bean);
    
    // -- add the line to the result after parsing it
    result += naming.parse();
    
    // -- return the result
    return result;
}

// ============================================================================
// == COPY LINE FOR CLONE METHOD
// ============================================================================

function createToString(field) {
    if (field.reference != null)
        return _createToString(field.reference, true);
    else 
        return _createToString(column, false);
}

function _createToString(field, reference) {
    // -- create the result
    var result = "";
    
    naming = util.createParameterString(
        "+ \"- :{identifier@lc@dc} = \" + :{identifier@lc@dc}"
    );
    
    if (reference) {
        naming.addParameter("identifier", convert.translateTo("java", field.parent.name));
    } else {
        naming.addParameter("identifier", convert.translateTo("java", field.name));
    }
    
    // -- add the line to the result after parsing it
    result += naming.parse();
    
    // -- return the result
    return result;
}

// ============================================================================
// == GETTER
// ============================================================================

function createGetter(field) {
    if (field.reference == null)
        // -- get the java type
        type = util.getJavaType(field);
        
        if (type == "Boolean")
        	return createBooleanGetter(field, "");
        else
        	return _createGetter(field, false);
    else
    	return _createGetter(field.reference, true);
        
}

function _createGetter(field, reference) {
    // -- create the result object
    var result = "";
    
    // -- create the naming
    naming = util.createParameterString(
        ":{accessModifier} :{type} get:{identifier@lc@ca}() {\n" +
        "\treturn :{identifier@lc};\n" +
        "\t}"
    );
    
    if (field.readable)
        naming.addParameter("accessModifier", "public");
    else
        naming.addParameter("accessModifier", "protected");
    
    if (reference) {
        // -- get the reference type
        var userClass = script.execute("default.beanClass", field.parent.name);
        
        naming.addParameter("type", userClass);
        naming.addParameter("identifier", field.parent.name);
    } else {
        naming.addParameter("type", util.getJavaType(field));
        naming.addParameter("identifier", field.name);
    }

    // -- add the line to the result after parsing it
    result += naming.parse();
    
    // -- return the result
    return result;
}

function createBooleanGetter(field, suffix) {
    // -- create the result object
    var result = "";
    
    // -- create the naming
    naming = util.createParameterString(
        ":{accessModifier} Boolean is:{identifier@lc@ca}:{suffix}() {\n" +
        "\treturn :{identifier@lc}:{suffix};\n" +
        "\t}"
    );
    
    if (field.readable)
        naming.addParameter("accessModifier", "public");
    else
        naming.addParameter("accessModifier", "protected");
    
    naming.addParameter("type", util.getJavaType(field));
   	naming.addParameter("identifier", field.name);
    naming.addParameter("suffix", suffix);

    // -- add the line to the result after parsing it
    result += naming.parse();
    
    // -- return the result
    return result;
}

// ============================================================================
// == SETTER
// ============================================================================

function createSetter(field) {
    if (field.reference == null)
    	return _createSetter(field, false);
    else
        return _createSetter(field.reference, true);
}

function _createSetter(field, reference) {
    // -- create the result object
    var result = "";
    
    // -- create the naming
    naming = util.createParameterString(
        ":{accessModifier} void set:{identifier@lc@ca}(:{type} newValue) {\n" +
        "\t:{type} oldValue = get:{identifier@lc@ca}();\n" +
        "\tif (changeSupport != null) \n" +
        "\t\tfirePropertyChanged(PROPERTY_:{identifier@uc}, oldValue, newValue);\n" +
        "\t:{identifier@lc}Initialized = true;\n" +
        "\t}"
    );
    
    if (field.writable)
        naming.addParameter("accessModifier", "public");
    else
        naming.addParameter("accessModifier", "protected");
    
    if (reference) {
        // -- get the reference type
        var userClass = script.execute("default.beanClass", field.parent.name);
        
        naming.addParameter("type", userClass);
        naming.addParameter("identifier", field.parent.name);
    } else {
        naming.addParameter("type", util.getJavaType(field));
        naming.addParameter("identifier", field.name);
    }

    // -- add the line to the result after parsing it
    result += naming.parse();
    
    // -- return the result
    return result; 
}

function keyColumnString(item) {
    var result = "";
    var counter = 0;
    
    var iter = item.keys.iterator();
    
    while (iter.hasNext()) {
        key = iter.next();

        if (counter++ > 0)
            result += ", ";
            
        result += util.getJavaType(key);
        result += " ";
        result += "?{" + key.name + "@lc}";
    }
    
    return result;
}